@@ -668,6 +668,19 @@ latter case, the loop can be safely re-started.
 Effectively blocks the graphic output for all widgets.
 The output can be restored with C<unlock()>.
 
+=item load_font FONTNAME
+
+Registers font resource in system-specific format. The resource is
+freed after prgram ends.
+
+Notes for win32: To add a font whose information comes from several
+resource files, point FONTNAME to a string with the file names separated by a
+C<|> - for example, C< abcxxxxx.pfm | abcxxxxx.pfb >.
+
+Notes for unix: available only when Prima is compiled with fontconfig and Xft .
+
+Returns number of font resources added.
+
 =item open_help TOPIC
 
 Opens the help viewer window with TOPIC string in 
