@@ -62,6 +62,8 @@ package
     ict; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# Image conversion types
 package 
     is; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# Image statistics types
+package 
+    ist; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# Image scaling types
 package 
     am; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# Icon auto masking
 package 
@@ -92,6 +94,8 @@ package
     ps; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# paint states
 package 
     scr; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# scroll() results
+package 
+    dbt; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# DeviceBitmap types
 
 1;
 
