@@ -1,30 +1,3 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  $Id$
-#
 package fontdlg;
 
 =pod 
@@ -51,13 +24,11 @@ Note the left-mouse drag effect from a font screen widget.
 
 
 use strict;
+use warnings;
 use Carp;
 use Prima;
-use Prima::Classes;
 use Prima::Application name => "Font Dialog";
-use Prima::Lists;
-use Prima::Sliders;
-use Prima::Buttons;
+use Prima qw(Lists Sliders Buttons FileDialog);
 use Encode;
 
 # try to use perl5.8 glyph names
@@ -181,11 +152,18 @@ $w = Prima::MainWindow-> create( text => "Font Window",
 );
 
 $displayRes = ($w-> resolution)[1];
-for ( sort { $a-> {name} cmp $b-> {name}} @{$::application-> fonts})
-{
-	$fontList{$_-> {name}} = $_;
-	push ( @fontItems, $_-> {name});
-}
+
+my $load_fonts = sub {
+	%fontList = ();
+	@fontItems = ();
+	for ( sort { $a-> {name} cmp $b-> {name}} @{$::application-> fonts})
+	{
+		$fontList{$_-> {name}} = $_;
+		push ( @fontItems, $_-> {name});
+	}
+};
+
+$load_fonts->();
 
 $w-> insert( ListBox =>
 	name   => "NameList",
@@ -371,6 +349,56 @@ onClick   => sub {
 			$p-> text_out( 'face name           : '.$cachedFacename, 2, $sd); $sd -= $fh;
 		},
 	);
+
+	my %wf = ( %$f, direction => 0 );
+	my @gsize= ( $f->maximalWidth + 10 , $f->height + 10 );
+	my $glyph = $ww-> insert( Widget => 
+		text => ' ',
+		font => \%wf,
+		size => \@gsize,
+		origin => [ $ww-> width - $gsize[0] - 10, $ww-> height - $gsize[1] - 10 ],
+		growMode => gm::GrowLoX|gm::GrowLoY,
+		backColor => cl::White,
+		#buffered => 1,
+		name    => 'Glyph',
+		textOutBaseline => 0,
+		onPaint => sub {
+			my $self = shift;
+			$self-> clear;
+			my $C = $self-> text;
+
+			my ( $a, $b, $c ) = @{ $self->get_font_abc( ord($C), ord($C), utf8::is_utf8($C)) };
+			my ( $d, $e, $f ) = @{ $self->get_font_def( ord($C), ord($C), utf8::is_utf8($C)) };
+			
+			my $w = (( $a < 0 ) ? 0 : $a) + $b + (( $c < 0 ) ? 0 : $c);
+			my $h = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f);
+			$w = ( $self-> width  - $w ) / 2;
+			$h = ( $self-> height - $h ) / 2;
+			$self-> translate($w, $h);
+
+			my $dx = 0;
+			my $dy = 0;
+			$dx -= $a if $a < 0;
+			$dy -= $d if $d < 0;
+
+			my $fh = $self-> font->height;
+			$self-> text_out( $C, $dx, $dy );
+
+			$dx = abs($a);
+			$dy = abs($d);
+			$self-> linePattern(lp::Dot);
+			$self-> line($dx, 0, $dx, $self->height);
+			$dx = (( $a < 0 ) ? 0 : $a) + $b + (( $c < 0 ) ? 0 : $c) - abs($c);
+			$self-> line($dx, 0, $dx, $self->height);
+			$self-> line(-$w, $self->font->descent, $self->width, $self->font->descent);
+
+			$self-> line(0, $dy, $self->width, $dy);
+			$dy = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f) - abs($f);
+			$self-> line(0, $dy, $self->width, $dy);
+		},
+	);
+
+
 	my @ranges = ([]);
 	for ( @{$w-> Example-> get_font_ranges}) {
 		( 2 > scalar @{$ranges[-1]}) ?
@@ -404,25 +432,26 @@ onClick   => sub {
 				}
 				$self-> hint( $pretty );
 				$self-> hintVisible(1);
+				$glyph->text(chr($c));
+				$glyph->repaint;
 			}
 		},
 		onDrawItem => sub {
-			my ($self, $canvas, $itemIndex, $x, $y, $x2, $y2, $selected, $focused) = @_;
+			my ($self, $canvas, $itemIndex, $x, $y, $x2, $y2, $selected, $focused, $prelight) = @_;
 			$canvas-> line( $x, $y, $x2, $y);
 			$canvas-> line( $x2+1, $y, $x2+1, $y2);
 			my @cs;
-			if ( $focused) {
+			if ( $focused || $prelight) {
 				@cs = ( $canvas-> color, $canvas-> backColor);
-				$canvas-> set( 
-					color => $canvas-> hiliteColor, 
-					backColor => $canvas-> hiliteBackColor
-				);
+				my $fo = $focused ? $canvas-> hiliteColor : $canvas-> color ;
+				my $bk = $focused ? $canvas-> hiliteBackColor : $canvas-> backColor ;
+				$canvas-> set( color => $fo, backColor => $bk );
 			}
-			$canvas-> clear( $x, $y + 1, $x2, $y2);
+			$self-> draw_item_background( $canvas, $x, $y + 1, $x2, $y2, $prelight );
 			if ( defined( my $c = $charmap{$itemIndex} )) {
 				$canvas-> text_out( chr($c), $x + $ih / 4, $y + $ih / 4);
 			}
-			$canvas-> set( color => $cs[0], backColor => $cs[1]) if $focused;
+			$canvas-> set( color => $cs[0], backColor => $cs[1]) if $focused || $prelight;
 		},
 	);
 	$l-> count( $count);
@@ -430,6 +459,28 @@ onClick   => sub {
 },
 );
 
+$w-> insert( Button =>
+origin => [ 180, 348],
+size   => [ 32, 32],
+text   => '+',
+hint   => 'Add new font',
+selectable => 0,
+name   => 'Load',
+onClick   => sub {
+	my $d = Prima::FileDialog-> new;
+	my $f = $d->execute;
+	return unless defined $f;
+	my $error = '';
+	local $SIG{__WARN__} = sub { $error = ": @_" };
+	unless ($::application->load_font($d->fileName)) {
+		$error =~ s/at examples.*//;
+		return Prima::message("Error loading font$error");
+	}
+
+	$load_fonts->();
+	$w->NameList->items(\@fontItems);
+});
+
 
 my $csl = $w-> insert( CircularSlider =>
 	origin      => [ 370, 348],
