@@ -199,15 +199,22 @@ img_put( Handle dest, Handle src, int dstX, int dstY, int srcX, int srcY, int ds
       int type     = PImage( src)-> type;
       void *self   = PImage( src)-> self; 
       RGBColor palette[2];
+      Byte *p;
+
+      PIcon( src)-> data = ( PIcon( src)-> maskType == imbpp8) ?
+	 CIcon( src)-> convert_mask( src, imbpp1) :
+         PIcon( src)-> mask;
       memcpy( palette, PImage( src)-> palette, 6);
       memcpy( PImage( src)-> palette, stdmono_palette, 6);
       PImage( src)-> self     =  CImage;
-      PImage( src)-> type     =  imbpp1 | imGrayScale; 
-      PImage( src)-> data     =  PIcon( src)-> mask;
+      PImage( src)-> type     =  imBW;
       PImage( src)-> lineSize =  PIcon( src)-> maskLine;
       PImage( src)-> dataSize =  PIcon( src)-> maskSize;
-      PImage( src)-> palSize  =  2;
+      PImage( src)-> palSize  = 2;
+      p = PIcon( src)-> data;
       img_put( dest, src, dstX, dstY, srcX, srcY, dstW, dstH, srcW, srcH, ropAndPut);
+      if ( PIcon( src)-> maskType == imbpp8)
+         free( PIcon( src)-> data );
       PImage( src)-> self     = self;
       PImage( src)-> type     = type;
       PImage( src)-> data     = data; 
@@ -547,7 +554,7 @@ img_bar( Handle dest, int x, int y, int w, int h, int rop, void * color)
       }
       if ( lmask ) *data = (lsave & lmask) | (*data & ~lmask);
       if ( rmask ) data[blt_bytes-1] = (rsave & rmask) | (data[blt_bytes-1] & ~rmask);
-      data -= lineSize;
+      data += lineSize;
    }
 }
 
