@@ -6,7 +6,7 @@
 #define POLLUTE_NAME_SPACE 1
 #endif
 
-#define PRIMA_CORE_VERSION 2016053001
+#define PRIMA_CORE_VERSION 2016063001
 
 #define PRIMA_VERSION_BOOTCHECK \
     if(apc_get_core_version()!=PRIMA_CORE_VERSION) \
@@ -1588,8 +1588,6 @@ typedef struct _ObjectOptions_ {
    unsigned optTabStop             : 1;
    unsigned optScaleChildren       : 1;
    unsigned optUTF8_helpContext    : 1;
-   unsigned optUTF8_hint           : 1;
-   unsigned optUTF8_text           : 1;
    unsigned optPreserveType        : 1;   /* Image */
    unsigned optAutoPopup           : 1;   /* Popup */
    unsigned optActive              : 1;   /* Timer */
@@ -1829,6 +1827,8 @@ SV(CanUTF8_Input)
 SV(CanUTF8_Output)
 #define   svCompositeDisplay 32
 SV(CompositeDisplay)
+#define   svLayeredWidgets  33
+SV(LayeredWidgets)
 END_TABLE(sv,UV)
 #undef SV
 
@@ -2034,7 +2034,7 @@ typedef struct {
 
 extern Bool
 apc_widget_create( Handle self, Handle owner, Bool syncPaint,
-                   Bool clipOwner, Bool transparent, ApiHandle parentHandle);
+                   Bool clipOwner, Bool transparent, ApiHandle parentHandle, Bool layered);
 
 extern Bool
 apc_widget_begin_paint( Handle self, Bool insideOnPaint);
@@ -2072,6 +2072,9 @@ apc_widget_get_handle( Handle self);
 extern Rect
 apc_widget_get_invalid_rect( Handle self);
 
+extern Bool
+apc_widget_get_layered( Handle self);
+
 extern Handle
 apc_widget_get_z_order( Handle self, int zOrderId);
 
@@ -2590,13 +2593,18 @@ typedef enum {
 #define ropNotSrcXor  ropNotXor    /* dest ^= !src */
 #define ropNotDestXor ropNotXor    /* dest  = !dest ^ src */
 
+typedef enum {
+   ropSrcOver = 0, /* save value as ropCopy, to serve as a default */
+   ropSrcCopy
+} ROP2;
+
 #define ROP(const_name) CONSTANT(rop,const_name)
 
 START_TABLE(rop,UV)
 ROP(Blackness) ROP(NotOr) ROP(NotSrcAnd) ROP(NotPut) ROP(NotDestAnd)
 ROP(Invert) ROP(XorPut) ROP(NotAnd) ROP(AndPut) ROP(NotXor) ROP(NoOper)
 ROP(NotSrcOr) ROP(CopyPut) ROP(NotDestOr) ROP(OrPut) ROP(Whiteness)
-ROP(NotSrcXor) ROP(NotDestXor)
+ROP(NotSrcXor) ROP(NotDestXor) ROP(SrcOver) ROP(SrcCopy)
 END_TABLE(rop,UV)
 #undef ROP
 
@@ -2943,9 +2951,21 @@ apc_image_get_error_message( char *errorMsgBuf, int bufLen);
 extern ApiHandle
 apc_image_get_handle( Handle self);
 
+/* text wrap options */
+#define DBT(const_name) CONSTANT(dbt,const_name)
+START_TABLE(dbt,UV)
+#define dbtBitmap         0    /* 1-bit BW bitmap */
+DBT(Bitmap)
+#define dbtPixmap         1    /* more-than-1 bit color bitmap */
+DBT(Pixmap)
+#define dbtLayered        2    /* ARGB bitmap */
+DBT(Layered)
+END_TABLE(dbt,UV)
+#undef DBT
+
 
 extern Bool
-apc_dbm_create( Handle self, Bool monochrome);
+apc_dbm_create( Handle self, int type);
 
 extern Bool
 apc_dbm_destroy( Handle self);
@@ -3213,6 +3233,9 @@ apc_gp_get_clip_rect( Handle self);
 extern PFontABC
 apc_gp_get_font_abc( Handle self, int firstChar, int lastChar, Bool unicode);
 
+extern PFontABC
+apc_gp_get_font_def( Handle self, int firstChar, int lastChar, Bool unicode);
+
 extern unsigned long *
 apc_gp_get_font_ranges( Handle self, int * count);
 
@@ -3384,7 +3407,7 @@ extern PFont
 apc_font_default( PFont font);
 
 extern int
-apc_font_load( const char* filename);
+apc_font_load( Handle self, char* filename);
 
 extern Bool
 apc_font_pick( Handle self, PFont source, PFont dest);
