@@ -12,6 +12,9 @@
 #include <X11/Xatom.h>
 #include <X11/Xresource.h>
 #include <X11/cursorfont.h>
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+#include <X11/extensions/Xrender.h>
+#endif
 #ifdef HAVE_X11_EXTENSIONS_SHAPE_H
 #include <X11/extensions/shape.h>
 #endif
@@ -112,10 +115,12 @@ typedef struct _PrimaXImage
 } PrimaXImage;
 
 
-#define CACHE_AUTODETECT 0
-#define CACHE_BITMAP     1
-#define CACHE_PIXMAP     2
-#define CACHE_LOW_RES    3
+#define CACHE_AUTODETECT     0
+#define CACHE_BITMAP         1
+#define CACHE_PIXMAP         2
+#define CACHE_LOW_RES        3
+#define CACHE_LAYERED        4
+#define CACHE_LAYERED_ALPHA  5
 
 typedef struct {
    int type;
@@ -460,6 +465,13 @@ prima_debug( const char *format, ...);
 #define Xdebug if (pguts->debug & DEBUG_XRDB) _debug
 #define _F_DEBUG_PITCH(x) ((x==fpDefault)?"default":(x==fpFixed?"fixed":"variable"))
 
+typedef struct 
+{
+   unsigned int red_shift, green_shift, blue_shift, alpha_shift;
+   unsigned int red_range, green_range, blue_range, alpha_range;
+   unsigned int red_mask,  green_mask,  blue_mask,  alpha_mask;
+} RGBABitDescription, *PRGBABitDescription;
+
 typedef struct _UnixGuts
 {
    /* Event management */
@@ -488,6 +500,7 @@ typedef struct _UnixGuts
    /* Graphics */
    struct gc_head               bitmap_gc_pool;
    struct gc_head               screen_gc_pool;
+   struct gc_head               argb_gc_pool;
    GC                                   menugc;
    TAILQ_HEAD(,_drawable_sys_data)      paintq;
    PHash                                ximages;
@@ -550,6 +563,7 @@ typedef struct _UnixGuts
    int                          machine_byte_order;
    int                          idepth; /* image depth; can be 32 if depth == 24 */
    int                          qdepth; /* image depth for querying */
+   int                          argb_depth; /* image depth for xrender RGBA */
    struct {
       long XDrawArcs;
       long XDrawLines;
@@ -583,6 +597,7 @@ typedef struct _UnixGuts
    int                          xft_xrender_major_opcode;
    Bool                         xft_no_antialias;
    Bool				randr_extension;
+   Bool				render_extension;
    struct MsgDlg               *message_boxes;
    XWindow                      grab_redirect;
    Handle                       grab_widget;
@@ -597,6 +612,11 @@ typedef struct _UnixGuts
    XWindow                      root;
    XVisualInfo                  visual;
    int                          visualClass;
+   XVisualInfo                  argb_visual;
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   XRenderPictFormat *          argb_pic_format;
+   XRenderPictFormat *          argb_compat_format;
+#endif
    MainColorEntry *             palette;
    int *                        mappingPlace;
    unsigned long                monochromeMap[2];
@@ -610,11 +630,12 @@ typedef struct _UnixGuts
    Bool                         useDithering;
    Bool                         privateColormap;
    Colormap                     defaultColormap;
+   Colormap                     argbColormap;
    FillPattern *                ditherPatterns;
    Point                        displaySize;
    long                         wm_event_timeout;
-   int                          red_shift, green_shift, blue_shift;
-   int                          red_range, green_range, blue_range;
+   RGBABitDescription           screen_bits;
+   RGBABitDescription           argb_bits;
    Point                        ellipseDivergence;
    int                          appLock;
    XGCValues                    cursor_gcv;
@@ -730,6 +751,7 @@ typedef struct _drawable_sys_data
       unsigned force_flush              : 1;
       unsigned grab                 	: 1;
       unsigned has_icon                 : 1;
+      unsigned layered                  : 1;
       unsigned iconic                   : 1;
       unsigned mapped			: 1;
       unsigned modal                    : 1;
@@ -759,16 +781,21 @@ typedef struct _drawable_sys_data
    TAILQ_HEAD(,configure_event_pair)    configure_pairs;
    Byte * palette;
    int borderIcons;
+   XVisualInfo * visual;
 #ifdef USE_XFT
    XftDraw  * xft_drawable;
    uint32_t * xft_map8;
    double     xft_font_cos;
    double     xft_font_sin;
 #endif
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   Picture    argb_picture;
+#endif
 } DrawableSysData, *PDrawableSysData;
 
 #define XF_ENABLED(x)   ((x)->flags.enabled)
 #define XF_IN_PAINT(x)  ((x)->flags.paint)
+#define XF_LAYERED(x)  ((x)->flags.layered)
 #define XFLUSH          if (XX->flags.force_flush) XFlush(DISP)
 
 #define MenuTimerMessage   1021
@@ -893,6 +920,7 @@ typedef union _unix_sys_data
 #define DEFCC		PClipboardSysData selfxx = C(self)
 #define XX		selfxx
 #define WHEEL_DELTA	120
+#define GET_RGBA_DESCRIPTION X(self)->flags.layered ? &guts. argb_bits : &guts. screen_bits
 
 typedef U8 ColorComponent;
 
@@ -984,18 +1012,18 @@ prima_create_icon_pixmaps( Handle bw_icon, Pixmap *xor, Pixmap *and);
 extern ImageCache*
 prima_create_image_cache( PImage img, Handle drawable, int type);
 
-void
+extern Bool
 prima_put_ximage( XDrawable win, GC gc, PrimaXImage *i,
                   int src_x, int src_y, int dst_x, int dst_y,
                   int width, int height);
 
-Bool
+extern Bool
 prima_query_image( Handle self, XImage * image);
 
-Bool
+extern Bool
 prima_std_query_image( Handle self, Pixmap px);
 
-Pixmap
+extern Pixmap
 prima_std_pixmap( Handle self, int type);
 
 extern void
@@ -1053,7 +1081,7 @@ extern void
 prima_ximage_event( XEvent*);
 
 extern PrimaXImage*
-prima_prepare_ximage( int width, int height, Bool bitmap);
+prima_prepare_ximage( int width, int height, int format);
 
 extern Bool
 prima_free_ximage( PrimaXImage *i);
@@ -1221,6 +1249,9 @@ prima_xft_get_font_ranges( Handle self, int * count);
 extern PFontABC
 prima_xft_get_font_abc( Handle self, int firstChar, int lastChar, Bool unicode);
 
+extern PFontABC
+prima_xft_get_font_def( Handle self, int firstChar, int lastChar, Bool unicode);
+
 extern PCachedFont
 prima_xft_get_cache( PFont font);
          
@@ -1233,15 +1264,18 @@ prima_xft_parse( char * ppFontNameSize, Font * font);
 extern void
 prima_xft_update_region( Handle self);
 
+extern int
+prima_xft_load_font( char * fontName );
+
 #endif
 
 #ifdef WITH_GTK2
-Display*
+extern Display*
 prima_gtk_init( void);
 
-Bool
+extern Bool
 prima_gtk_done( void);
 
-char *
+extern char *
 prima_gtk_openfile( char * params);
 #endif
