@@ -38,7 +38,6 @@ typedef HANDLE SOCKETHANDLE;
 #define REG_STORAGE                      "SOFTWARE\\Perl\\Prima"
 #define MAXREGLEN                        1024
 
-
 #define WM_WRITE_TO_LOG                   ( WM_USER + 0)
 #define WM_PRIMA_CREATE                   ( WM_USER + 1)
 #define WM_POSTAL                         ( WM_USER + 2)
@@ -59,6 +58,7 @@ typedef HANDLE SOCKETHANDLE;
 #define WM_HASMATE                        ( WM_USER + 18)
 #define WM_FILE                           ( WM_USER + 19)
 #define WM_CROAK                          ( WM_USER + 20)
+#define WM_REPAINT_LAYERED                ( WM_USER + 21)
 #define WM_TERMINATE                      ( WM_USER + 99)
 #define WM_FIRST_USER_MESSAGE             ( WM_USER +100)
 #define WM_LAST_USER_MESSAGE              ( WM_USER +900)
@@ -129,7 +129,10 @@ typedef struct _HandleOptions_ {
    unsigned aptVisible              : 1;       // visibility flag
    unsigned aptTaskList             : 1;       // Window flag - set if in task list
    unsigned aptDeviceBitmap         : 1;       // == kind_of( CDeviceBitmap)
-   unsigned aptBitmap               : 1;       // == kind_of( CImage)
+   unsigned aptBitmap               : 1;       // buffered widget
+   unsigned aptImage                : 1;       // == kind_of( CImage)
+   unsigned aptIcon                 : 1;       // == kind_of( CIcon)
+   unsigned aptPrinter              : 1;       // == kind_of( CPrinter)
    unsigned aptExtraFont            : 1;       // extra font styles ( angle, shear) has been applied
    unsigned aptDCChangeLock         : 1;       // locks SelectObject() calls
    unsigned aptEnabled              : 1;       // enabled flag
@@ -137,6 +140,9 @@ typedef struct _HandleOptions_ {
    unsigned aptTextOutBaseline      : 1;       // gp text drawing flag
    unsigned aptWinPosDetermined     : 1;       // 0 when size is set, but position is not
    unsigned aptOnTop                : 1;       // HWND_TOPMOST is set
+   unsigned aptLayered              : 1;       // WS_EX_LAYERED
+   unsigned aptRepaintPending       : 1;       // for optLayered
+   unsigned aptMovePending          : 1;       // for optLayered
 } HandleOptions;
 
 typedef struct _WinGuts
@@ -186,6 +192,7 @@ typedef struct _WinGuts
     HANDLE         socketThread;       // thread id
     Bool           socketPostSync;     // semaphore
     Bool           dont_xlate_message; // one-time stopper to TranslateMessage() call
+    int            utf8_prepend_0x202D;// newer windows do automatic bidi conversion, this is to cancel it
     WCHAR *      (*alloc_utf8_to_wchar_visual)(const char*,int,int*);
 } WinGuts, *PWinGuts;
 
@@ -211,6 +218,39 @@ typedef struct _FileData
    int          type;
 } FileData;
 
+typedef struct _XLOGPALETTE {
+   WORD         palVersion;
+   WORD         palNumEntries;
+   PALETTEENTRY palPalEntry[ 256];
+} XLOGPALETTE, *PXLOGPALETTE;
+
+typedef struct _XBITMAPINFO {
+   BITMAPINFOHEADER bmiHeader;
+   RGBQUAD          bmiColors[ 256];
+} XBITMAPINFO, *PXBITMAPINFO;
+
+#define BM_NONE    0
+#define BM_BITMAP  1
+#define BM_PIXMAP  2
+#define BM_LAYERED 3
+#define BM_AUTO    4
+
+typedef struct _ImageCache
+{
+      int         cacheType;
+      XBITMAPINFO rawHeader;
+      Byte*       rawBits;
+      Bool        freeBits;
+      HBITMAP     bitmap; /* copy of sys bm, if any */
+} ImageCache;
+
+typedef struct _ImageData
+{
+      HRGN        imgCachedRegion;
+      uint32_t*   argbBits;
+      ImageCache  cache;
+} ImageData;
+
 typedef struct _PrinterData
 {
    PRINTER_INFO_2 ppi;
@@ -291,17 +331,6 @@ typedef struct _DCFont
 } DCFont, *PDCFont;
 
 
-typedef struct _XLOGPALETTE {
-   WORD         palVersion;
-   WORD         palNumEntries;
-   PALETTEENTRY palPalEntry[ 256];
-} XLOGPALETTE, *PXLOGPALETTE;
-
-typedef struct _XBITMAPINFO {
-   BITMAPINFOHEADER bmiHeader;
-   RGBQUAD          bmiColors[ 256];
-} XBITMAPINFO, *PXBITMAPINFO;
-
 typedef struct _ItemRegRec {
   int   cmd;
   void *item;
@@ -390,6 +419,7 @@ typedef struct _DrawableData
    Handle         lastMenu;                // last menu activated by WM_INITMENU or WM_INITMENUPOPUP
    Point          extraBounds;             // used in region calculations
    Point          extraPos;                // used in region calculations
+   Point          layeredPos;              // delayed layered window positioning
 
    /* Other class-specific data */
    union {
@@ -397,7 +427,7 @@ typedef struct _DrawableData
       WindowData    window;
       PrinterData   prn;
       FileData      file;
-      HRGN          imgCachedRegion;      // Image specific field
+      ImageData     image;
    } s;
 } DrawableData, *PDrawableData;
 
@@ -472,12 +502,13 @@ typedef struct _MusClkRec {
    !dsys(handle)options.aptClipOwner                                                    \
 )
 
-#define palette_create image_make_bitmap_palette
+#define palette_create image_create_palette
 
 extern Bool         appDead;
 extern Bool         debug;
 extern DIBMONOBRUSH bmiHatch;
 extern PHash        fontMan;
+extern PHash        myfontMan;
 extern int          FONTSTRUCSIZE;
 extern WinGuts      guts;
 extern PHash        imageMan;
@@ -525,15 +556,13 @@ extern Handle       hwnd_frame_top_level( Handle self);
 extern void         hwnd_leave_paint( Handle self);
 extern Handle       hwnd_to_view( HWND win);
 extern Handle       hwnd_top_level( Handle self);
-extern void         image_destroy_cache( Handle self);
-extern Handle       image_enscreen( Handle image, Handle screen);
-extern BITMAPINFO * image_get_binfo( Handle img, XBITMAPINFO * bi);
-extern HBITMAP      image_make_bitmap_handle( Handle img, HPALETTE palette);
-extern HPALETTE     image_make_bitmap_palette( Handle img);
+extern Bool         hwnd_repaint_layered( Handle self, Bool now);
 extern HICON        image_make_icon_handle( Handle img, Point size, Point * hotSpot);
 extern void         image_query_bits( Handle self, Bool forceNewImage);
-extern Bool         image_screenable( Handle image, Handle screen, int * bitCount);
-extern Bool         image_set_cache( Handle from, Handle self);
+extern HBITMAP      image_create_bitmap( Handle self, HPALETTE pal, XBITMAPINFO * bitmapinfo, int bm_type);
+extern HPALETTE     image_create_palette( Handle self);
+extern void         image_destroy_cache( Handle self);
+extern BITMAPINFO*  image_fill_bitmap_info( Handle self, XBITMAPINFO * bi, int bm_type);
 extern void         mod_free( BYTE * modState);
 extern BYTE *       mod_select( int mod);
 extern Bool         palette_change( Handle self);
