@@ -168,7 +168,8 @@ Widget_init( Handle self, HV * profile)
       var-> virtualSize = my-> get_size( self);
    var-> geomSize = var-> virtualSize;
 
-   {
+   int geometry = pget_i(geometry);
+   if ( geometry == gtGrowMode ) {
       Bool x = 0, y = 0;
       if ( pget_B( centered)) { x = 1; y = 1; };
       if ( pget_B( x_centered) || ( var-> growMode & gmXCenter)) x = 1;
@@ -179,7 +180,7 @@ Widget_init( Handle self, HV * profile)
    opt_assign( optPackPropagate, pget_B( packPropagate));
    my-> set_packInfo( self, pget_sv( packInfo));
    my-> set_placeInfo( self, pget_sv( placeInfo));
-   my-> set_geometry( self, pget_i( geometry));
+   my-> set_geometry( self, geometry);
    
    my-> set_shape       ( self, pget_H(  shape));
    my-> set_visible     ( self, pget_B( visible));
@@ -206,53 +207,65 @@ Widget_init( Handle self, HV * profile)
    }
 }
 
-
 void
 Widget_update_sys_handle( Handle self, HV * profile)
 {
    dPROFILE;
    enter_method;
    Handle    owner;
-   Bool      clipOwner;
+   Bool      clipOwner, layered;
    ApiHandle parentHandle;
    if (!(
        pexist( owner) ||
        pexist( syncPaint) ||
        pexist( clipOwner) ||
+       pexist( layered) ||
        pexist( transparent)
     )) return;
    
    owner        = pexist( owner)        ? pget_H( owner)        : var-> owner;
    clipOwner    = pexist( clipOwner)    ? pget_B( clipOwner)    : my-> get_clipOwner( self);
    parentHandle = pexist( parentHandle) ? pget_i( parentHandle) : apc_widget_get_parent_handle( self);
+   layered      = pexist( layered)      ? pget_i( layered)      : my-> get_layered(self);
 
    if ( parentHandle) {
       if (( owner != application) && clipOwner) 
          croak("Cannot accept 'parentHandle' for non-application child and clip-owner widget");
    }
+   if ( layered) {
+      if (( owner != application) && clipOwner) 
+         croak("Cannot accept 'layered' for non-application child and clip-owner widget");
+      if ( var-> widgets. count > 0)
+         croak("Cannot accept 'layered' while there are children on this widget");
+   } else {
+      if ( CWidget(owner)-> get_layered(owner))
+         croak("Cannot be a child of a layered widget");
+   }
    
    if ( !apc_widget_create( self,
       owner,
       pexist( syncPaint)  ? pget_B( syncPaint)  : my-> get_syncPaint( self),
       clipOwner,
       pexist( transparent) ? pget_B( transparent): my-> get_transparent( self),
-      parentHandle
+      parentHandle,
+      layered
    ))
      croak( "Cannot create widget");
    pdelete( transparent);
    pdelete( syncPaint);
    pdelete( clipOwner);
    pdelete( parentHandle);
+   pdelete( layered);
 }
 
 void
 Widget_done( Handle self)
 {
-   free( var-> text);
+   if ( var-> text ) SvREFCNT_dec( var-> text );
    var-> text = nil;
    apc_widget_destroy( self);
    free( var-> helpContext);
-   free( var-> hint);
+   if ( var-> hint) SvREFCNT_dec( var-> hint );
    var-> helpContext = nil;
    var-> hint = nil;
 
@@ -664,8 +677,9 @@ void Widget_handle_event( Handle self, PEvent event)
       case cmMouseEnter:
         my-> notify( self, "<siP", "MouseEnter", event-> pos. mod, event -> pos. where);
         objCheck;
-        if ( application && is_opt( optShowHint) && ((( PApplication) application)-> options. optShowHint) && var-> hint[0])
+        if ( application && is_opt( optShowHint) && ((( PApplication) application)-> options. optShowHint) && var-> hint && SvCUR(var-> hint))
         {
+		
            PApplication app = ( PApplication) application;
            app-> self-> set_hint_action( application, self, true, true);
         }
@@ -846,7 +860,7 @@ void Widget_handle_event( Handle self, PEvent event)
                   oldP. y != event-> gen. P. y)) {
                my-> notify( self, "<sPP", "Move", oldP, event-> gen. P);
                objCheck;
-               if ( var-> growMode & gmCenter) 
+               if ( var->geometry == gtGrowMode && var-> growMode & gmCenter) 
                   my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
             }
          }
@@ -895,7 +909,7 @@ void Widget_handle_event( Handle self, PEvent event)
               n-> gen. P. y = n-> gen. R. top    = event-> gen. P. y;
            }
         SIZE_EVENT:;  
-           if ( var-> growMode & gmCenter) 
+           if ( var->geometry == gtGrowMode && var-> growMode & gmCenter) 
                my-> set_centered( self, var-> growMode & gmXCenter, var-> growMode & gmYCenter);
            if ( !event-> gen. B)
                my-> first_that( self, (void*) Widget_size_notify, &event-> gen. R);
@@ -1707,7 +1721,7 @@ Widget_growMode( Handle self, Bool set, int growMode)
    var-> growMode = growMode;
    if ( var-> growMode & gmXCenter) x = true;
    if ( var-> growMode & gmYCenter) y = true;
-   if ( x || y) my-> set_centered( self, x, y);
+   if ( var-> geometry == gtGrowMode && (x || y)) my-> set_centered( self, x, y);
    return var-> growMode;
 }
 
@@ -1738,7 +1752,7 @@ Widget_hintVisible( Handle self, Bool set, int hintVisible)
    wantVisible = ( hintVisible != 0);
    if ( wantVisible == PApplication( application)-> hintVisible) return false;
    if ( wantVisible) {
-      if ( strlen( var-> hint) == 0) return false;
+      if ( SvCUR( var-> hint) == 0) return false;
       if ( hintVisible > 0) PApplication(application)-> hintActive = -1; /* immediate */
    }
    CApplication( application)-> set_hint_action( application, self, wantVisible, false);
@@ -1817,6 +1831,9 @@ Widget_set_centered( Handle self, Bool x, Bool y)
    Point mysize  = my-> get_size ( self);
    Point mypos   = my-> get_origin( self);
    Point delta   = {0,0};
+
+   if ( !x && !y ) return;
+
    if ( parent == application ) {
         int i, nrects = 0;
         Rect2 *best = nil, *rects = apc_application_get_monitor_rects( application, &nrects);
@@ -1866,13 +1883,13 @@ Widget_set_popup_font( Handle self, Font font)
 void
 Widget_on_paint( Handle self, SV * canvas)
 {
-	int i;
+   int i;
    dSP;
    ENTER;
    SAVETMPS;
    PUSHMARK( sp);
    XPUSHs( canvas);
-	for ( i = 0; i < 4; i++)
+   for ( i = 0; i < 4; i++)
       XPUSHs( sv_2mortal( newSViv( -1)));
    PUTBACK;
    PERL_CALL_METHOD( "clear", G_DISCARD);
@@ -2368,30 +2385,39 @@ Widget_hint( Handle self, Bool set, SV *hint)
    if ( set) {
       if ( var-> stage > csFrozen) return nilSV;
       my-> first_that( self, (void*)hint_notify, (void*)hint);
-      free( var-> hint);
-      var-> hint = nil;
-      var-> hint = duplicate_string( SvPV_nolen( hint));
-      opt_assign( optUTF8_hint, prima_is_utf8_sv(hint));
+      if ( var-> hint ) SvREFCNT_dec( var-> hint );
+      var-> hint = hint;
+      if ( var-> hint ) SvREFCNT_inc( var-> hint );
       if ( application && (( PApplication) application)-> hintVisible &&
            (( PApplication) application)-> hintUnder == self)
       {
-         SV   * hintText   = my-> get_hint( self);
          Handle hintWidget = (( PApplication) application)-> hintWidget;
-         if ( strlen( var-> hint) == 0) 
+         if ( SvLEN( var-> hint) == 0) 
             my-> set_hintVisible( self, 0);
          if ( hintWidget) 
-            CWidget(hintWidget)-> set_text( hintWidget, hintText);
-         sv_free( hintText);
+            CWidget(hintWidget)-> set_text( hintWidget, my-> get_hint( self));
       }
       opt_clear( optOwnerHint);
    } else {
-      hint = newSVpv( var-> hint ? var-> hint : "", 0);
-      if ( is_opt( optUTF8_hint)) SvUTF8_on( hint);
-      return hint;
+      return newSVsv(var->hint);
    }
    return nilSV;
 }
 
+Bool
+Widget_layered( Handle self, Bool set, Bool layered)
+{
+   HV * profile;
+   enter_method;
+   if ( !set)
+      return apc_widget_get_layered( self);
+   profile = newHV();
+   pset_i( layered, layered);
+   my-> set( self, profile);
+   sv_free(( SV *) profile);
+   return false;
+}
+
 int
 Widget_left( Handle self, Bool set, int left)
 {
@@ -2965,16 +2991,13 @@ Widget_text( Handle self, Bool set, SV *text)
 {
    if ( set) {
       if ( var-> stage > csFrozen) return nilSV;
-      free( var-> text);
-      var-> text = nil;
-      var-> text = duplicate_string( SvPV_nolen( text));
-      opt_assign( optUTF8_text, prima_is_utf8_sv(text));
+      if ( var-> text ) SvREFCNT_dec( var-> text );
+      var-> text = text;
+      if ( var-> text ) SvREFCNT_inc( var-> text );
+      return nilSV;
    } else {
-      text = newSVpv( var-> text ? var-> text : "", 0);
-      if ( is_opt( optUTF8_text)) SvUTF8_on( text);
-      return text;
+      return newSVsv(var->text);
    }
-   return nilSV;
 }
 
 int
