@@ -253,6 +253,13 @@ init_x11( char * error_buf )
       if ( XRRQueryExtension( DISP, &dummy, &dummy))
          guts. randr_extension = true;
    }	 
+#endif
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   {
+      int dummy;
+      if ( XRenderQueryExtension( DISP, &dummy, &dummy))
+         guts. render_extension = true;
+   }	 
 #endif
    XrmInitialize();
    guts.db = get_database();
@@ -306,6 +313,7 @@ init_x11( char * error_buf )
    TAILQ_INIT( &guts.peventq);
    TAILQ_INIT( &guts.bitmap_gc_pool);
    TAILQ_INIT( &guts.screen_gc_pool);
+   TAILQ_INIT( &guts.argb_gc_pool);
 
    guts. currentFocusTime = CurrentTime;
    guts. windows = hash_create();
@@ -553,6 +561,7 @@ window_subsystem_done( void)
 
    free_gc_pool(&guts.bitmap_gc_pool);
    free_gc_pool(&guts.screen_gc_pool);
+   free_gc_pool(&guts.argb_gc_pool);
    prima_done_color_subsystem();
    free( guts. clipboard_formats);
 
@@ -624,6 +633,7 @@ apc_application_create( Handle self)
    XX-> origin. y = 0;
    XX-> ackSize = XX-> size = apc_application_get_size( self);
    XX-> owner = nilHandle;
+   XX-> visual = &guts. visual;
 
    XX-> flags. clip_owner = 1;
    XX-> flags. sync_paint = 0;
