@@ -54,7 +54,7 @@ void
 prima_get_gc( PDrawableSysData selfxx)
 {
    XGCValues gcv;
-   Bool bitmap;
+   Bool bitmap, layered;
    struct gc_head *gc_pool;
 
    if ( XX-> gc && XX-> gcl) return;
@@ -65,13 +65,14 @@ prima_get_gc( PDrawableSysData selfxx)
    }   
 
    bitmap = XT_IS_BITMAP(XX) ? true : false;
-   gc_pool = bitmap ? &guts.bitmap_gc_pool : &guts.screen_gc_pool;
+   layered = XF_LAYERED(XX) ? true : false;
+   gc_pool = bitmap ? &guts.bitmap_gc_pool : ( layered ? &guts.argb_gc_pool : &guts.screen_gc_pool );
    XX->gcl = TAILQ_FIRST(gc_pool);
    if (XX->gcl)
       TAILQ_REMOVE(gc_pool, XX->gcl, gc_link);
    if (!XX->gcl) {
       gcv. graphics_exposures = false;
-      XX-> gc = XCreateGC( DISP, bitmap ? XX-> gdrawable : guts. root, GCGraphicsExposures, &gcv);
+      XX-> gc = XCreateGC( DISP, (bitmap || layered) ? XX-> gdrawable : guts. root, GCGraphicsExposures, &gcv);
       XCHECKPOINT;
       if (( XX->gcl = alloc1z( GCList))) 
          XX->gcl->gc = XX-> gc;
@@ -82,14 +83,15 @@ prima_get_gc( PDrawableSysData selfxx)
 void
 prima_release_gc( PDrawableSysData selfxx)
 {
-   Bool bitmap;
+   Bool bitmap, layered;
    struct gc_head *gc_pool;
 
    if ( XX-> gc) {
       if ( XX-> gcl == nil)
          warn( "UAG_011: internal error");
       bitmap = XT_IS_BITMAP(XX) ? true : false;
-      gc_pool = bitmap ? &guts.bitmap_gc_pool : &guts.screen_gc_pool;
+      layered = XF_LAYERED(XX) ? true : false;
+      gc_pool = bitmap ? &guts.bitmap_gc_pool : ( layered ? &guts.argb_gc_pool : &guts.screen_gc_pool );
       if ( XX-> gcl) 
          TAILQ_INSERT_HEAD(gc_pool, XX->gcl, gc_link);
       XX->gcl = nil;
@@ -152,7 +154,7 @@ prima_prepare_drawable_for_painting( Handle self, Bool inside_on_paint)
          XX-> bsize. y = h = XX-> size. y;
       }
       if ( w <= 0 || h <= 0) goto Unbuffered;
-      XX-> gdrawable = XCreatePixmap( DISP, XX-> udrawable, w, h, guts.depth);
+      XX-> gdrawable = XCreatePixmap( DISP, XX-> udrawable, w, h, XX->visual->depth);
       XCHECKPOINT;
       if (!XX-> gdrawable) goto Unbuffered;
       XX-> gcv. ts_x_origin = -r.x;
@@ -188,11 +190,11 @@ Unbuffered:
    } else {
       XX-> paint_dashes = malloc(1);
       if ( XX-> ndashes < 0) {
-	 XX-> paint_dashes[0] = '\0';
-	 XX-> paint_ndashes = 0;
+         XX-> paint_dashes[0] = '\0';
+         XX-> paint_ndashes = 0;
       } else {
-	 XX-> paint_dashes[0] = '\1';
-	 XX-> paint_ndashes = 1;
+         XX-> paint_dashes[0] = '\1';
+         XX-> paint_ndashes = 1;
       }
       XX-> line_style = LineSolid;
    }
@@ -931,14 +933,15 @@ color_to_pixel( Handle self, Color color, int depth)
    } else if ( guts.palSize > 0 ) {
       pv = prima_color_find( self, color, -1, nil, RANK_FREE);
    } else {
+      PRGBABitDescription bd = GET_RGBA_DESCRIPTION;
       switch ( depth) {
       case 16:   
       case 24:   
       case 32:   
          pv = 
-            (((COLOR_R(color) << guts. red_range  ) >> 8) << guts.   red_shift) |
-            (((COLOR_G(color) << guts. green_range) >> 8) << guts. green_shift) |
-            (((COLOR_B(color) << guts. blue_range ) >> 8) << guts.  blue_shift);
+            (((COLOR_R(color) << bd-> red_range  ) >> 8) << bd->   red_shift) |
+            (((COLOR_G(color) << bd-> green_range) >> 8) << bd-> green_shift) |
+            (((COLOR_B(color) << bd-> blue_range ) >> 8) << bd->  blue_shift);
          if ( guts.machine_byte_order != guts.byte_order)  
             switch( depth) {
             case 16:
@@ -1206,16 +1209,18 @@ apc_gp_get_pixel( Handle self, int x, int y)
          } else 
             c = guts.palette[pixel]. composite;
       } else {
-         int r, g, b, rmax, gmax, bmax;
+         PRGBABitDescription bd = GET_RGBA_DESCRIPTION;
+         int r, g, b, rmax, gmax, bmax, depth;
          rmax = gmax = bmax = 0xff;
+         depth = XF_LAYERED(XX) ? guts. argb_visual. depth : guts. idepth;
          switch ( guts. idepth) {
          case 16:
             p32 = *(( uint16_t*)(im-> data));
             if ( guts.machine_byte_order != guts.byte_order) 
                p32 = REVERSE_BYTES_16(p32);
-            rmax = 0xff & ( 0xff << ( 8 - guts. red_range));
-            gmax = 0xff & ( 0xff << ( 8 - guts. green_range));
-            bmax = 0xff & ( 0xff << ( 8 - guts. blue_range));
+            rmax = 0xff & ( 0xff << ( 8 - bd-> red_range));
+            gmax = 0xff & ( 0xff << ( 8 - bd-> green_range));
+            bmax = 0xff & ( 0xff << ( 8 - bd-> blue_range));
             goto COMP;
          case 24:   
             p32 = (im-> data[0] << 16) | (im-> data[1] << 8) | im-> data[2];
@@ -1227,9 +1232,9 @@ apc_gp_get_pixel( Handle self, int x, int y)
             if ( guts.machine_byte_order != guts.byte_order) 
                p32 = REVERSE_BYTES_32(p32);
          COMP:   
-            r = ((((p32 & guts. visual. red_mask)   >> guts. red_shift) << 8) >> guts. red_range) & 0xff;
-            g = ((((p32 & guts. visual. green_mask) >> guts. green_shift) << 8) >> guts. green_range) & 0xff;
-            b = ((((p32 & guts. visual. blue_mask)  >> guts. blue_shift) << 8) >> guts. blue_range) & 0xff;
+            r = ((((p32 & bd-> red_mask)   >> bd->red_shift) << 8)   >> bd-> red_range) & 0xff;
+            g = ((((p32 & bd-> green_mask) >> bd->green_shift) << 8) >> bd-> green_range) & 0xff;
+            b = ((((p32 & bd-> blue_mask)  >> bd->blue_shift) << 8)  >> bd-> blue_range) & 0xff;
             if ( r == rmax ) r = 0xff;
             if ( g == gmax ) g = 0xff;
             if ( b == bmax ) b = 0xff;
@@ -1543,7 +1548,10 @@ apc_gp_set_region( Handle self, Handle mask)
    XX-> flags. xft_clip = 0;
 #ifdef USE_XFT
    if ( XX-> xft_drawable) prima_xft_update_region( self);
-#endif   
+#endif
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   if ( XX-> argb_picture ) XRenderSetPictureClipRegion(DISP, XX->argb_picture, region);
+#endif
    return true;
 }
 
@@ -1869,6 +1877,9 @@ apc_gp_get_back_color( Handle self)
 int
 apc_gp_get_bpp( Handle self)
 {
+   DEFXX;
+   if ( XT_IS_BITMAP(XX)) return 1;
+   if ( XF_LAYERED(XX)) return guts. argb_pic_format-> depth;
    return guts. depth;
 }
 
@@ -1965,10 +1976,6 @@ apc_gp_get_font_abc( Handle self, int firstChar, int lastChar, Bool unicode)
 
    if ( self) {
       DEFXX;
-      /*
-      if (!XX-> font) apc_gp_set_font( self, &PDrawable( self)-> font);
-      if (!XX-> font) return nil;
-      */
 #ifdef USE_XFT
       if ( XX-> font-> xft)
          return prima_xft_get_font_abc( self, firstChar, lastChar, unicode);
@@ -1980,16 +1987,110 @@ apc_gp_get_font_abc( Handle self, int firstChar, int lastChar, Bool unicode)
    return abc;
 }
 
+PFontABC
+prima_xfont2def( Handle self, int first, int last)
+{
+   DEFXX;
+   XCharStruct *max;
+   Pixmap pixmap;
+   GC gc;
+   XGCValues gcv;
+   int i, j, k, w, h, ls;
+   PFontABC ret;
+   XImage *xi;
+
+   if ( !( ret = malloc( sizeof(FontABC) * ( last - first + 1 ) )))
+      return nil;
+   bzero( ret, sizeof(FontABC) * ( last - first + 1 ));
+
+   max = &XX-> font-> fs-> max_bounds;
+   w  = max-> width * 3;
+   h  = max-> descent + max-> ascent;
+   ls = (( w + 31) / 32) * 4;
+   w  = ls * 8;
+   pixmap = XCreatePixmap( DISP, guts. root, w, h, 1);
+   gcv. graphics_exposures = false;
+   gc = XCreateGC( DISP, pixmap, GCGraphicsExposures, &gcv);
+   XSetFont( DISP, gc, XX-> font-> id);
+
+   for ( i = 0; i <= last - first; i++) {
+      XChar2b ch;
+      ch. byte1 = (first + i) / 256;
+      ch. byte2 = (first + i) % 256;
+      XSetForeground( DISP, gc, 0);
+      XFillRectangle( DISP, pixmap, gc, 0, 0, w, h);
+      XSetForeground( DISP, gc, 1);
+      XDrawString16( DISP, pixmap, gc, 10, h - XX->font->font. descent, &ch, 1);
+
+      if (!(xi = XGetImage( DISP, pixmap, 0, 0, w, h, 1, XYPixmap))) {
+      	 free( ret );
+	 ret = nil;
+	 break;
+      }
+      /*
+      for ( j = 0; j < h; j++) {
+         int k, l;
+         for ( k = 0; k < ls; k++) {
+            Byte * p = (Byte*)xi-> data + j * xi-> bytes_per_line + k;
+            printf(".");
+            for ( l = 0; l < 8; l++) {
+               int z = (*p) & ( 1 << l );
+               printf("%s", z ? "*" : " ");
+            }
+         }
+         printf("\n");
+      }
+      */
+      for ( j = 0; j < h; j++) {
+         Byte * p = (Byte*)xi-> data + j * xi-> bytes_per_line;
+         for ( k = 0; k < ls; k++, p++) {
+	     if ( *p != 0 ) {
+                ret[i]. c = j;
+                goto FOUND_C;
+	     }
+	 }
+      }
+      FOUND_C:
+      for ( j = h - 1; j >= 0; j--) {
+         Byte * p = (Byte*)xi-> data + j * xi-> bytes_per_line;
+         for ( k = 0; k < ls; k++, p++) {
+	     if ( *p != 0 ) {
+                ret[i]. a = h - j - 1;
+                goto FOUND_A;
+	     }
+	 }
+      }
+      FOUND_A:
+      if ( ret[i].a != 0 || ret[i].c != 0)
+         ret[i]. b = h - ret[i]. a - ret[i]. c;
+      XDestroyImage( xi);
+   }
+
+   XFreeGC( DISP, gc);
+   XFreePixmap( DISP, pixmap);
+
+   return ret; 
+}
+
+PFontABC
+apc_gp_get_font_def( Handle self, int firstChar, int lastChar, Bool unicode)
+{
+   PFontABC abc;
+   DEFXX;
+#ifdef USE_XFT
+   if ( XX-> font-> xft)
+      return prima_xft_get_font_def( self, firstChar, lastChar, unicode);
+#endif
+   abc = prima_xfont2def( self, firstChar, lastChar);
+   return abc;
+}
+
 unsigned long *
 apc_gp_get_font_ranges( Handle self, int * count)
 {
    DEFXX;
    unsigned long * ret = nil;
    XFontStruct * fs;
-   /*
-   if (!XX-> font) apc_gp_set_font( self, &PDrawable( self)-> font);
-   if (!XX-> font) return nil;
-   */
 #ifdef USE_XFT
    if ( XX-> font-> xft)
       return prima_xft_get_font_ranges( self, count);
@@ -2319,6 +2420,9 @@ apc_gp_set_clip_rect( Handle self, Rect clipRect)
 #ifdef USE_XFT
    if ( XX-> xft_drawable) prima_xft_update_region( self);
 #endif   
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   if ( XX-> argb_picture ) XRenderSetPictureClipRegion(DISP, XX->argb_picture, region);
+#endif
    return true;
 }
 
