@@ -349,6 +349,7 @@ apc_component_destroy( Handle self)
       free( XX-> q_class_name);
       XX-> q_class_name = nil;
    }
+
    free( PComponent( self)-> sysData);
    PComponent( self)-> sysData = nil;
    X_WINDOW = nilHandle;
@@ -449,11 +450,11 @@ prima_update_cursor( Handle self)
 	 guts. cursor_save = XCreatePixmap( DISP, XX-> udrawable,
 					    guts. cursor_pixmap_size. x,
 					    guts. cursor_pixmap_size. y,
-					    guts. depth);
+					    XX-> visual-> depth);
 	 guts. cursor_xor  = XCreatePixmap( DISP, XX-> udrawable,
 					    guts. cursor_pixmap_size. x,
 					    guts. cursor_pixmap_size. y,
-					    guts. depth);
+					    XX-> visual-> depth);
       }
 
       prima_get_gc( XX);
@@ -1159,7 +1160,8 @@ apc_sys_get_value( int v)  /* XXX one big XXX */
    case svColorPointer:         return 0;
    case svCanUTF8_Input:        return 1;
    case svCanUTF8_Output:       return 1;
-   case svCompositeDisplay:     return 0; /* XXX detect compiz */
+   case svCompositeDisplay:     return guts. argb_visual. visual != NULL; /* XXX doesn't check if composition is actually available */
+   case svLayeredWidgets:       return guts. argb_visual. visual != NULL;
    default:
       return -1;
    }
