@@ -177,20 +177,83 @@ flush_events( Display * disp, XEvent * ev, Handle self)
    return win == X(self)-> client || win == X_WINDOW;
 }
 
+typedef struct _ViewProfile {
+  Point    pos;
+  Point    size;
+  Bool     visible;
+  Bool     focused;
+  Handle   capture;
+} ViewProfile, *PViewProfile;
+
+static void
+get_view_ex( Handle self, PViewProfile p)
+{
+  int i;
+  if ( !p) return;
+  p-> capture   = apc_widget_is_captured( self);
+  p-> pos       = apc_widget_get_pos( self);
+  p-> size      = apc_widget_get_size( self);
+  p-> focused   = apc_widget_is_focused( self);
+  p-> visible   = apc_widget_is_visible( self);
+}
+
+static void
+set_view_ex( Handle self, PViewProfile p)
+{
+  DEFXX;
+  if ( XX-> flags. falsely_hidden) {
+     if ( p-> size. x == 0) p-> size. x = 1;
+     if ( p-> size. y == 0) p-> size. y = 1;
+     XMoveResizeWindow( DISP, X_WINDOW, p-> pos.x, p->pos. y, p-> size.x, p-> size.y);
+  } else {
+     p-> pos. y = X(XX-> owner)-> size. y - p-> size. y - XX-> origin. y;
+     if ( XX-> parentHandle) {
+        XWindow cld;
+        XTranslateCoordinates( DISP, PWidget(XX-> owner)-> handle, XX-> parentHandle, p->pos.x, p->pos.y, &p->pos.x, &p->pos.y, &cld);
+     } 
+     XMoveResizeWindow( DISP, X_WINDOW, p-> pos.x, p->pos. y, p-> size.x, p-> size.y);
+     if ( p-> visible ) XMapWindow( DISP, X_WINDOW);
+  }
+
+  if ( p-> focused) apc_widget_set_focused( self);
+  if ( p-> capture) apc_widget_set_capture( self, 1, nilHandle);
+}
+
 Bool
 apc_widget_create( Handle self, Handle owner, Bool sync_paint,
-		   Bool clip_owner, Bool transparent, ApiHandle parentHandle)
+		   Bool clip_owner, Bool transparent, ApiHandle parentHandle, Bool layered)
 {
    DEFXX;
-   Bool reset;
-   Handle real_owner;
+   ViewProfile vprf;
+   Bool reparent, recreate;
+   Handle real_owner, old_parent;
    XWindow parent, old = X_WINDOW;
    XSetWindowAttributes attrs;
+   unsigned long valuemask;
 
-   reset = ( old != nilHandle) && ( 
+   if ( !guts. argb_visual. visual || guts. argb_visual. visualid == guts. visual. visualid)
+      layered = false;
+   XX-> visual = layered ? &guts. argb_visual : &guts. visual;
+
+   reparent = ( old != nilHandle) && ( 
       ( clip_owner != XX-> flags. clip_owner) ||
       ( parentHandle != XX-> parent)
    );
+   recreate = ( old != nilHandle) && (
+      ( layered != XX-> flags. layered )
+   );
+   if ( recreate ) {
+      reparent = false;
+      get_view_ex( self, &vprf);
+      if ( guts. currentMenu && PComponent( guts. currentMenu)-> owner == self) prima_end_menu();
+      CWidget( self)-> end_paint_info( self);
+      CWidget( self)-> end_paint( self);
+      hash_delete( guts.windows, (void*)&old, sizeof(old), false);
+      XDestroyWindow( DISP, old);
+      XCHECKPOINT;
+   }
+
+   old_parent = ( old != nilHandle ) ? XX->parent : nilHandle;
 
    XX-> flags. transparent = !!transparent;
    XX-> type.drawable = true;
@@ -212,6 +275,7 @@ apc_widget_create( Handle self, Handle owner, Bool sync_paint,
 
    XX-> flags. clip_owner = !!clip_owner;
    XX-> flags. sync_paint = !!sync_paint;
+   XX-> flags. layered    = !!layered;
 
    attrs. event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
       | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask | PointerMotionMask
@@ -224,9 +288,12 @@ apc_widget_create( Handle self, Handle owner, Bool sync_paint,
       ? SouthWestGravity : NorthWestGravity;
    attrs. colormap = guts. defaultColormap;
 
-   if ( reset) {
+   if ( reparent) {
       Point pos = PWidget(self)-> pos;
       XEvent dummy_ev;
+
+      if ( old_parent == parent ) return true;
+
       if ( guts. currentMenu && PComponent( guts. currentMenu)-> owner == self) prima_end_menu();
       CWidget( self)-> end_paint_info( self);
       CWidget( self)-> end_paint( self);
@@ -240,7 +307,8 @@ apc_widget_create( Handle self, Handle owner, Bool sync_paint,
       
       XChangeWindowAttributes( DISP, X_WINDOW, CWWinGravity, &attrs);
       XReparentWindow( DISP, X_WINDOW, parent, pos. x, 
-         X(owner)-> size.y - pos. y - X(self)-> size. y);
+         X(real_owner)-> size.y - pos. y - X(self)-> size. y);
+
       XX-> ackOrigin = pos;
       XX-> ackSize   = XX-> size;
       XX-> flags. mapped = XX-> flags. want_visible;
@@ -248,31 +316,53 @@ apc_widget_create( Handle self, Handle owner, Bool sync_paint,
       return true;
    }   
 
-   if ( old != nilHandle) return true;
+   valuemask = 
+      0
+      /* | CWBackPixmap */
+      /* | CWBackPixel */
+      /* | CWBorderPixmap */
+      /* | CWBorderPixel */
+      /* | CWBitGravity */
+      | CWWinGravity
+      /* | CWBackingStore */
+      /* | CWBackingPlanes */
+      /* | CWBackingPixel */
+      | CWOverrideRedirect
+      /* | CWSaveUnder */
+      | CWEventMask
+      /* | CWDontPropagate */
+      | CWColormap 
+      /* | CWCursor */
+   ;
+
+   if ( layered ) {
+      valuemask |= CWBackPixel | CWBorderPixel;
+      if ( !guts. argbColormap ) guts. argbColormap = XCreateColormap( DISP, guts. root, guts. argb_visual. visual, AllocNone);
+      attrs. colormap = guts. argbColormap;
+      attrs. background_pixel = 0;
+      attrs. border_pixel = 0;
+   }
 
    XX-> client = X_WINDOW = XCreateWindow( DISP, parent,
-                             0, 0, 1, 1, 0, guts. visual. depth,
-                             InputOutput, VISUAL,
-                             0
-                             /* | CWBackPixmap */
-                             /* | CWBackPixel */
-                             /* | CWBorderPixmap */
-                             /* | CWBorderPixel */
-                             /* | CWBitGravity */
-                             | CWWinGravity
-                             /* | CWBackingStore */
-                             /* | CWBackingPlanes */
-                             /* | CWBackingPixel */
-                             | CWOverrideRedirect
-                             /* | CWSaveUnder */
-                             | CWEventMask
-                             /* | CWDontPropagate */
-                             | CWColormap 
-                             /* | CWCursor */
-                             , &attrs);
-   if (!X_WINDOW)
-      return false;
+                             0, 0, 1, 1, 0, XX-> visual-> depth,
+                             InputOutput, XX-> visual-> visual,
+                             valuemask, &attrs);
    XCHECKPOINT;
+   if (!X_WINDOW) {
+      warn("error creating window");
+      return false;
+   }
+
+   if (recreate) {
+      Point pos = PWidget(self)-> pos;
+      set_view_ex( self, &vprf);
+      XX-> gdrawable = XX-> udrawable = X_WINDOW;
+      XX-> ackOrigin = pos;
+      XX-> ackSize   = XX-> size;
+      XX-> flags. mapped = XX-> flags. want_visible;
+      hash_store( guts.windows, &X_WINDOW, sizeof(X_WINDOW), (void*)self);
+      return true;
+   }
 
    XX-> size. x = XX-> size. y = 
    XX-> ackOrigin. x = XX-> ackOrigin. y = 
@@ -328,6 +418,10 @@ apc_widget_begin_paint( Handle self, Bool inside_on_paint)
    if ( guts. dynamicColors && inside_on_paint) prima_palette_free( self, false);
    prima_no_cursor( self);
    prima_prepare_drawable_for_painting( self, inside_on_paint);
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   if ( XF_LAYERED(XX) )
+      XX->argb_picture = XRenderCreatePicture( DISP, XX->gdrawable, guts. argb_pic_format, 0, NULL);
+#endif
    if ( useRPDraw) {
       Handle owner = PWidget(self)->owner;
       Point po = apc_widget_get_pos( self);
@@ -410,7 +504,6 @@ apc_widget_destroy( Handle self)
       TAILQ_REMOVE( &guts.paintq, XX, paintq_link);
       XX-> flags. paint_pending = false;
    }
-   if ( XX-> recreateData) free( XX-> recreateData);
    if ( XX-> invalid_region) {
       XDestroyRegion( XX-> invalid_region);
       XX-> invalid_region = nil;
@@ -448,6 +541,12 @@ apc_widget_end_paint( Handle self)
 {
    DEFXX;
    XX-> flags. force_flush = 0;
+#ifdef HAVE_X11_EXTENSIONS_XRENDER_H
+   if ( XF_LAYERED(XX) && XX->argb_picture ) {
+      XRenderFreePicture( DISP, XX->argb_picture);
+      XX->argb_picture = 0;
+   }
+#endif
    prima_cleanup_drawable_after_painting( self);
    prima_update_cursor( self);
    return true;
@@ -509,6 +608,12 @@ apc_widget_get_invalid_rect( Handle self)
    return ret;
 }
 
+Bool
+apc_widget_get_layered( Handle self)
+{
+   return X(self)-> flags. layered;
+}
+
 Point
 apc_widget_get_pos( Handle self)
 {
@@ -555,14 +660,16 @@ apc_widget_get_shape( Handle self, Handle mask)
    if ( XX-> shape_extent. x == 0 || XX-> shape_extent. y == 0)
       return false;
 
-   r = rc = XShapeGetRectangles( DISP, XX-> client, ShapeBounding, &count, &ordering);
+   r = rc = XShapeGetRectangles( DISP, X_WINDOW, ShapeBounding, &count, &ordering);
   
    CImage(mask)-> create_empty( mask, XX-> shape_extent. x, XX-> shape_extent. y, imBW);
    CImage(mask)-> begin_paint( mask); 
+   XSetForeground( DISP, X(mask)-> gc, 0);
+   XFillRectangle( DISP, X(mask)->gdrawable, X(mask)->gc, 0, 0, XX->shape_extent.x, XX->shape_extent.y);
    XSetForeground( DISP, X(mask)-> gc, 1);
    for ( i = 0; i < count; i++, r++) 
       XFillRectangle( DISP, X(mask)-> gdrawable, X(mask)-> gc, 
-          r-> x - XX-> shape_offset. x, r-> y - XX-> shape_offset. y, 
+          r-> x - XX-> shape_offset. x, r-> y - XX-> shape_offset. y,
           r-> width, r-> height);
    XFree( rc);
    CImage(mask)-> end_paint( mask);  
@@ -1027,7 +1134,7 @@ apc_widget_set_shape( Handle self, Handle mask)
    XGCValues gcv;
    ImageCache * cache;
    int i;
-   Byte * data;
+   XRectangle xr;
 
    if ( !guts. shape_extension) return false; 
    
@@ -1041,38 +1148,30 @@ apc_widget_set_shape( Handle self, Handle mask)
    }
    
    img = PImage(mask);
-   data = img-> data;
-   for ( i = 0; i < img-> dataSize; i++, data++) *data = ~*data;
    cache = prima_create_image_cache(img, nilHandle, CACHE_BITMAP);
    if ( !cache) return false;
-   px = XCreatePixmap(DISP, guts. root, img->w, img->h + XX-> menuHeight, 1);
+   px = XCreatePixmap(DISP, guts. root, img->w, img->h, 1);
    gcv. graphics_exposures = false;
    gc = XCreateGC(DISP, px, GCGraphicsExposures, &gcv);
-   if ( XX-> menuHeight > 0) {
-      XSetForeground( DISP, gc, 1);
-      XFillRectangle( DISP, px, gc, 0, 0, img-> w, XX-> menuHeight);
-   }
    XSetForeground( DISP, gc, 0);
-   prima_put_ximage(px, gc, cache->image, 0, 0, 0, XX-> menuHeight, img->w, img->h);
+   prima_put_ximage(px, gc, cache->image, 0, 0, 0, 0, img->w, img->h);
    XFreeGC( DISP, gc);
    /*
       XXX This static shape approach doesn't work when menuHeight is dynamically changed.
           Need to implement something more elaborated.
      */
-   XShapeCombineMask( DISP, X_WINDOW, ShapeBounding, 0, 0, px, ShapeSet);
-   XShapeOffsetShape( DISP, X_WINDOW, ShapeBounding, 0, XX-> size. y - img-> h);
-   if ( XX-> client != X_WINDOW) {
-      XShapeCombineMask( DISP, XX-> client, ShapeBounding, 0, 0, px, ShapeSet);
-      XShapeOffsetShape( DISP, XX-> client, ShapeBounding, 0, XX-> size. y - img-> h - XX-> menuHeight);
-   }
+   xr. x = 0;
+   xr. y = 0;
+   XShapeCombineMask( DISP, X_WINDOW, ShapeBounding, 0, XX->size.y - img->h + XX->menuHeight, px, ShapeSet);
+   xr. width  = XX->size.x;
+   xr. height = XX->size.y + XX->menuHeight;
+   XShapeCombineRectangles( DISP, X_WINDOW, ShapeBounding, 0, 0, &xr, 1, ShapeInvert, 0);
    XFreePixmap( DISP, px);
-   data = img-> data;
-   for ( i = 0; i < img-> dataSize; i++, data++) *data = ~*data;
    apc_image_update_change( mask);
    XX-> shape_extent. x = img-> w; 
    XX-> shape_extent. y = img-> h; 
    XX-> shape_offset. x = 0;
-   XX-> shape_offset. y = XX-> size. y + XX-> menuHeight - img-> h;
+   XX-> shape_offset. y = XX-> menuHeight;
    return true;
 #endif   
 }
@@ -1186,7 +1285,6 @@ apc_widget_set_size( Handle self, int width, int height)
    return true;
 }
 
-
 Bool
 apc_widget_set_rect( Handle self, int x, int y, int width, int height)
 {
@@ -1228,7 +1326,6 @@ apc_widget_set_rect( Handle self, int x, int y, int width, int height)
          &guts. grab_translate_mouse.x, &guts. grab_translate_mouse.y, &rx);
    }
 
-
    XX-> size. x = width;
    XX-> size. y = height;
 
