@@ -276,6 +276,8 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
       apc_window_task_listed( self, task_list);
       return true; 
    }
+   
+   XX-> visual = &guts. visual;
 
    /* create */
    attrs. event_mask = 0
@@ -309,7 +311,7 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
    attrs. colormap = guts. defaultColormap;
    X_WINDOW = XCreateWindow( DISP, guts. root,
 	                     0, 0, 1, 1, 0, guts. visual. depth,
-	                     InputOutput, VISUAL,
+	                     InputOutput, XX->visual->visual,
 	                     0
 	                     /* | CWBackPixmap */
 	                     /* | CWBackPixel */
@@ -361,7 +363,7 @@ apc_window_create( Handle self, Handle owner, Bool sync_paint, int border_icons,
 
    XX-> client = XCreateWindow( DISP, X_WINDOW,
 	                     0, 0, 1, 1, 0, guts. visual. depth,
-	                     InputOutput, VISUAL,
+	                     InputOutput, XX-> visual-> visual,
 	                     0
 	                     /* | CWBackPixmap */
 	                     /* | CWBackPixel */
@@ -612,9 +614,7 @@ apc_window_get_icon( Handle self, Handle icon)
    if ( !prima_std_query_image( icon, xor)) return false;
    
    if ( and) {
-      HV * profile = newHV();
-      Handle mask = Object_create( "Prima::Image", profile);
-      sv_free(( SV *) profile);
+      Handle mask = (Handle) create_object( "Prima::Image", "");
       CImage( mask)-> create_empty( mask, ax, ay, ( ad == 1) ? imBW : guts. qdepth);
       ret = prima_std_query_image( mask, and);
       if (( PImage( mask)-> type & imBPP) != 1)
@@ -951,10 +951,16 @@ prima_window_reset_menu( Handle self, int newMenuHeight)
       
 #ifdef HAVE_X11_EXTENSIONS_SHAPE_H
      if ( XX-> shape_extent. x != 0 || XX-> shape_extent. y != 0) {
-        int ny = XX-> size. y + XX-> menuHeight - XX-> shape_extent. y;
+        int ny = XX-> menuHeight;
         if ( XX-> shape_offset. y != ny) {
+           XRectangle xr;
            XShapeOffsetShape( DISP, X_WINDOW, ShapeBounding, 0, ny - XX-> shape_offset. y);
            XX-> shape_offset. y = ny;
+           xr. x = 0;
+           xr. y = 0;
+           xr. width  = XX->size.x;
+           xr. height = XX->menuHeight;
+           XShapeCombineRectangles( DISP, X_WINDOW, ShapeBounding, 0, 0, &xr, 1, ShapeUnion, 0);
         }
      }
 #endif     
@@ -1059,7 +1065,7 @@ apc_window_set_icon( Handle self, Handle icon)
          XSetForeground( DISP, gc, 0x00000000);
          prima_put_ximage( and, gc, X(icon)-> image_cache. icon, 0, 0, 0, 0, i-> w, i-> h);
       } else {
-         XSetForeground( DISP, gc, 0xffffffff);
+         XSetForeground( DISP, gc, guts. monochromeMap[1]);
          XFillRectangle( DISP, and, gc, 0, 0, i-> w + 1, i-> h + 1);
       }
       XFreeGC( DISP, gc);
