@@ -113,6 +113,7 @@ static PHash encodings    = nil;
 static PHash mono_fonts   = nil; /* family->mono font mapping */
 static PHash prop_fonts   = nil; /* family->proportional font mapping */
 static PHash mismatch     = nil; /* fonts not present in xft base */
+static PHash myfont_cache = nil; /* fonts loaded temporarily */
 static char  fontspecific[] = "fontspecific";
 static CharSetInfo * locale = nil;
 
@@ -231,6 +232,7 @@ prima_xft_init(void)
    mono_fonts   = hash_create();
    prop_fonts   = hash_create();
    encodings    = hash_create();
+   myfont_cache = hash_create();
    for ( i = 0; i < MAX_CHARSET; i++) {
       int length = 0;
       char upcase[256], *dest = upcase, *src = std_charsets[i].name;
@@ -252,6 +254,13 @@ prima_xft_init(void)
    FcCharSetDestroy( fcs_ascii);
 }
 
+static Bool
+remove_myfonts( void * f, int keyLen, void * key, void * dummy)
+{
+   unlink((char*) key);
+   return false;
+}   
+
 void
 prima_xft_done(void)
 {
@@ -264,6 +273,11 @@ prima_xft_done(void)
    hash_destroy( mismatch, false);
    hash_destroy( prop_fonts, true);
    hash_destroy( mono_fonts, true);
+   
+   hash_first_that( myfont_cache, (void*)remove_myfonts, nil, nil, nil); 
+   hash_destroy( myfont_cache, false);
+   myfont_cache = nil;
+
 }
 
 static unsigned short
@@ -1662,6 +1676,33 @@ prima_xft_get_font_abc( Handle self, int firstChar, int lastChar, Bool unicode)
    return abc;
 }
 
+PFontABC
+prima_xft_get_font_def( Handle self, int firstChar, int lastChar, Bool unicode)
+{
+   PFontABC abc;
+   int i, len = lastChar - firstChar + 1;
+   XftFont *font = X(self)-> font-> xft_base;
+
+   if ( !( abc = malloc( sizeof( FontABC) * len))) 
+      return nil;
+
+   for ( i = 0; i < len; i++) {
+      FcChar32 c = i + firstChar;
+      FT_UInt ft_index;
+      XGlyphInfo glyph;
+      if ( !unicode && c > 128) {
+         c = X(self)-> xft_map8[ c - 128];
+      }
+      ft_index = XftCharIndex( DISP, font, c);
+      XftGlyphExtents( DISP, font, &ft_index, 1, &glyph);
+      abc[i]. a = X(self)-> font-> font. descent - glyph. height + glyph. y; /* XXX yOff ? */
+      abc[i]. b = glyph. height;
+      abc[i]. c = X(self)-> font-> font. ascent - glyph. y;
+   }
+
+   return abc;
+}
+
 uint32_t *
 prima_xft_map8( const char * encoding)
 {
@@ -1710,6 +1751,41 @@ prima_xft_update_region( Handle self)
    }
 }
 
+int
+prima_xft_load_font( char* filename)
+{
+   int count = 0;
+   struct stat s;
+   FcPattern * font;
+   FcConfig * config;
+   FcFontSet *fonts;   
+   
+   /* -f $filename or die */
+   if (stat( filename, &s) < 0) {
+      warn("%s", strerror(errno));
+      return 0;
+   }
+
+#define FAIL(msg) { warn(msg); return 0; }
+
+   if (( s.st_mode & S_IFDIR) != 0) 
+      FAIL("Must not be a directory")
+   if ( !( font = FcFreeTypeQuery (filename, 0, NULL, &count))) 
+      FAIL("Format not recognized")
+   FcPatternDestroy (font);
+
+   if ( count == 0 ) 
+      FAIL("No fonts found in file")
+   if ( !(config = FcConfigGetCurrent())) 
+      FAIL("FcConfigGetCurrent error")
+   if ( !( fonts = FcConfigGetFonts(config, FcSetSystem))) 
+      FAIL("FcConfigGetFonts(FcSetSystem) error")
+   if ( !( FcFileScan(fonts, NULL, NULL, NULL, filename, FcFalse)))
+      FAIL("FcFileScan error")
+
+   return count;
+}
+
 #else
 #error Required: Xft version 2.1.0 or higher; fontconfig version 2.0.1 or higher. To compile without Xft, re-run 'perl Makefile.PL WITH_XFT=0'
 #endif /* USE_XFT */
