@@ -28,6 +28,7 @@ PHash   patMan       = nil; // pattern resource manager
 PHash   menuMan      = nil; // HMENU manager
 PHash   imageMan     = nil; // HBITMAP manager
 PHash   regnodeMan   = nil; // cache for apc_widget_user_profile
+PHash   myfontMan    = nil; // hash of calls to apc_font_load
 HPEN    hPenHollow;
 HBRUSH  hBrushHollow;
 PatResource hPatHollow;
@@ -68,10 +69,13 @@ window_subsystem_init( char * error_buf)
 
    guts. version  = GetVersion();
    GetVersionEx( &os);
-   guts. alloc_utf8_to_wchar_visual = 
+   guts. utf8_prepend_0x202D =
    	(( os.dwMajorVersion > 5) || (os.dwMajorVersion == 5 && os.dwMinorVersion > 1)) ?
-		alloc_utf8_to_wchar_visual :
-		alloc_utf8_to_wchar;
+		1 : 0;
+   guts. alloc_utf8_to_wchar_visual =
+        guts. utf8_prepend_0x202D ?
+           alloc_utf8_to_wchar_visual :
+           alloc_utf8_to_wchar;
    guts. mainThreadId = GetCurrentThreadId();
    guts. errorMode = SetErrorMode( SEM_FAILCRITICALERRORS);
    guts. desktopWindow = GetDesktopWindow();
@@ -118,6 +122,7 @@ window_subsystem_init( char * error_buf)
    menuMan    = hash_create();
    imageMan   = hash_create();
    regnodeMan = hash_create();
+   myfontMan  = hash_create();
    create_font_hash();
    {
       LOGBRUSH b = { BS_HOLLOW, 0, 0};
@@ -265,6 +270,11 @@ window_subsystem_set_option( char * option, char * value)
    return false;
 }
 
+static Bool myfont_cleaner( void * value, int keyLen, void * key, void * dummy) {
+   RemoveFontResource((LPCTSTR)key);
+   return false;
+}
+
 void
 window_subsystem_done()
 {
@@ -288,6 +298,9 @@ window_subsystem_done()
    hash_destroy( fontMan,    true);
    hash_destroy( stylusMan,  true);
    hash_destroy( regnodeMan, false);
+
+   hash_first_that( myfontMan, myfont_cleaner, nil, nil, nil);
+   hash_destroy( myfontMan,  false);
    DeleteObject( hPenHollow);
    DeleteObject( hBrushHollow);
    SetErrorMode( guts. errorMode);
@@ -841,6 +854,12 @@ AGAIN:
            ( list_index_of( &guts. transp, self) >= 0)
          )
          return 0;
+      if ( is_apt( aptLayered )) {
+         PAINTSTRUCT ps;
+         HDC hdc = BeginPaint(win, &ps);
+         EndPaint(win, &ps);
+         return 0;
+      }
       break;
    case WM_QUERYNEWPALETTE:
       return palette_change( self);
@@ -860,6 +879,9 @@ AGAIN:
    case WM_PRIMA_CREATE:
       ev. cmd = cmSetup;
       break;
+   case WM_REPAINT_LAYERED:
+      hwnd_repaint_layered( self, true );
+      break;
    case WM_SETFOCUS:
       if ( guts. focSysDialog) return 1;
       // dlg protect code - general case
