@@ -654,212 +654,6 @@ apc_gp_set_pixel( Handle self, int x, int y, Color color)
 }
 
 
-static Handle ctx_rop2R4[] = {
-  ropCopyPut      ,  SRCCOPY          ,
-  ropXorPut       ,  SRCINVERT        ,
-  ropAndPut       ,  SRCAND           ,
-  ropOrPut        ,  SRCPAINT         ,
-  ropNotPut       ,  NOTSRCCOPY       ,
-  ropNotDestAnd   ,  SRCERASE         ,
-  ropNotDestOr    ,  0x00DD0228       ,
-  ropNotSrcAnd    ,  0x00220326       ,
-  ropNotSrcOr     ,  MERGEPAINT       ,
-  ropNotXor       ,  0x00990066       ,
-  ropNotAnd       ,  0x007700E6       ,
-  ropNotOr        ,  NOTSRCERASE      ,
-  ropNoOper       ,  0x00AA0029       ,
-  ropBlackness    ,  BLACKNESS        ,
-  ropWhiteness    ,  WHITENESS        ,
-  ropInvert       ,  DSTINVERT        ,
-  endCtx
-};
-
-/*
-static void dc2screen( HDC dc, Handle self)
-{
-   HDC xdc = dc_alloc();
-   if ( !BitBlt( xdc, 0, 0, var w, var h, dc, 0, 0, SRCCOPY)) apiErr;
-   dc_free();
-}
-*/
-
-Bool
-apc_gp_stretch_image( Handle self, Handle image, int x, int y, int xFrom, int yFrom, int xDestLen, int yDestLen, int xLen, int yLen, int rop)
-{objCheck false;{
-   PIcon  i    = ( PIcon) image;
-   Handle deja = image;
-   int    ly   = sys lastSize. y;
-   HDC    xdc  = sys ps;
-   HPALETTE p1, p2 = nil, p3;
-   HBITMAP  b1;
-   HDC dc;
-   DWORD theRop;
-   Bool ok = true, db, dcmono = false;
-   POINT tr = {0, 0};
-
-   COLORREF oFore = 0, oBack = 0;
-
-   dobjCheck(image) false;
-   db = dsys( image) options. aptDeviceBitmap || i-> options. optInDraw;
-
-   if ( xLen == 0 || yLen == 0) return false;
-
-   // Determinig whether we have bitmap adapted for output or it's just bare bits
-   if ( db) {
-      if (
-          ( guts. displayBMInfo. bmiHeader. biBitCount == 8) &&
-          ( !is_apt( aptCompatiblePS)) &&
-          // ( is_apt( aptWinPS) || is_apt( aptBitmap))
-          is_apt( aptBitmap)
-         )
-      {
-         // There is a big uncertainity about 8-bit BitBlt's, based on fact that
-         // memory DCs aren't really compatible with screen DCs, - correct results
-         // could be guaranteed only when you Blt source DC = CreateCompatibleDC( dest DC),
-         // not when source = CCDC( thirdDC), dest = CCDC( thirdDC) - that's right, I mean it!
-         // or just SetDIBits; - latter is slow, stupid, memory-greedy - but surely can be trusted.
-         Bool ok;
-         Handle img      = ( Handle) create_object("Prima::Image", "");
-         HDC adc         = dsys( image) ps;
-         HBITMAP abitmap = dsys( image) bm;
-         dsys( img) ps   = dsys( image) ps;
-         dsys( img) bm   = dsys( image) bm;
-         image_query_bits( img, true);
-         dsys( img) ps   = adc;
-         dsys( img) bm   = abitmap;
-         ok = apc_gp_stretch_image( self, img, x, y, xFrom, yFrom, xDestLen, yDestLen, xLen, yLen, rop);
-         Object_destroy( img);
-         return ok;
-      }
-
-      dc = dsys( image) ps;
-      SetViewportOrgEx( dc, 0, 0, &tr);
-   } else {
-      if ( is_apt( aptCompatiblePS))
-         dc = CreateCompatibleDC( xdc);
-      else
-         dc = nil;
-      if ( dc) {
-         if ( dsys( image) bm == nil) {
-            image_destroy_cache( image); // if palette still exists
-            deja = image_enscreen( image, self);
-            if ( !image_set_cache( deja, image)) {
-	       // we're low on memory, reverting to StretchDIBits
-	       DeleteDC( dc);
-	       dc = nil;
-	    }
-         }
-      } else
-         deja = image_enscreen( image, self);
-   }
-
-   // actions for mono images
-   if ( dsys( image) options. aptDeviceBitmap) {
-      if ((( PDeviceBitmap) image)-> monochrome)
-         STYLUS_USE_TEXT( sys ps);
-   } else
-      dcmono = i-> options. optInDraw ?  ( dsys( image) bpp == 1) : (( i-> type & imBPP) == 1);
-
-   if ( dcmono) {
-      PRGBColor pal = i-> palette;
-      oFore  = GetTextColor( sys ps);
-      oBack  = GetBkColor( sys ps);
-      SetTextColor( sys ps,
-         i-> palSize > 0
-         ? RGB( pal[0].r, pal[0].g, pal[0].b)
-         : RGB( 0, 0, 0));
-      SetBkColor( sys ps,
-         i-> palSize > 1
-         ? RGB( pal[1].r, pal[1].g, pal[1].b)
-         : RGB( 0xff, 0xff, 0xff));
-   }
-
-   // if image is actually icon, drawing and-mask
-   if ( kind_of( deja, CIcon)) {
-      XBITMAPINFO xbi = {
-         { sizeof( BITMAPINFOHEADER), 0, 0, 1, 1, BI_RGB, 0, 0, 0, 2, 2},
-         { {0,0,0,0}, {255,255,255,0}}
-      };
-      xbi. bmiHeader. biWidth = i-> w;
-      xbi. bmiHeader. biHeight = i-> h;
-      if ( StretchDIBits(
-         xdc, x, ly - y - yDestLen, xDestLen, yDestLen, xFrom, yFrom, xLen, yLen,
-         i-> mask, ( BITMAPINFO*) &xbi, DIB_RGB_COLORS, SRCAND
-         ) == GDI_ERROR) {
-         ok = false;
-         apiErr;
-      }
-      theRop = SRCINVERT;
-   } else {
-      theRop = ctx_remap_def( rop, ctx_rop2R4, true, SRCCOPY);
-   }
-
-   // saving bitmap and palette ( if available) to both dc and xdc.
-   p3 = dsys( image) pal;
-
-   if ( p3 && !db && dc) 
-      p1 = SelectPalette( dc, p3, 0);
-   else
-      p1 = nil;
-
-   if ( db)
-      b1 = nil;
-   else {
-      if ( dc)
-         b1 = SelectObject( dc, dsys( image) bm);
-      else
-         b1 = nil;
-   }
-
-   if ( p3) {
-      p2 = SelectPalette( xdc, p3, 1);
-      RealizePalette( xdc);
-   }
-
-   //
-   if ( dc) {
-      if ( !StretchBlt( xdc, x, ly - y - yDestLen, xDestLen, yDestLen, dc,
-            xFrom, i-> h - yFrom - yLen, xLen, yLen, theRop)) {
-	 ok = false;
-         apiErr;
-      }
-   } else {
-      XBITMAPINFO xbi;
-      BITMAPINFO * bi = image_get_binfo( deja, &xbi);
-      if ( bi-> bmiHeader. biClrUsed > 0)
-         bi-> bmiHeader. biClrUsed = bi-> bmiHeader. biClrImportant = PImage(deja)-> palSize;
-      if ( StretchDIBits( xdc, x, ly - y - yDestLen, xDestLen, yDestLen,
-            xFrom, yFrom,
-            xLen, yLen, ((PImage) deja)-> data, bi,
-            DIB_RGB_COLORS, theRop) == GDI_ERROR) {
-         ok = false;
-         apiErr;
-      }
-   }
-
-   // restoring gdiobjects back
-   if ( p3) {
-      if ( p1) SelectPalette( dc,  p1, 1);
-   }
-   if ( p2) SelectPalette( xdc, p2, 1);
-   if ( b1) SelectObject( dc, b1);
-
-   if ( dcmono) {
-      SetTextColor( sys ps, oFore);
-      SetBkColor( sys ps, oBack);
-   }
-
-   if ( dc && tr. x != 0 && tr. y != 0) 
-      SetViewportOrgEx( dc, tr. x, tr. y, NULL);
-
-   if ( !db) {
-      if ( dc) DeleteDC( dc);
-      if ( deja != image) Object_destroy( deja);
-   }
-
-   return ok;
-}}
-
 Bool
 apc_gp_text_out( Handle self, const char * text, int x, int y, int len, Bool utf8 )
 {objCheck false;{
@@ -950,6 +744,149 @@ apc_gp_get_font_abc( Handle self, int first, int last, Bool unicode)
    return f1;
 }}
 
+/* extract vertical data from a bitmap */
+Bool
+gp_get_font_def_bitmap( Handle self, int first, int last, Bool unicode, PFontABC abc)
+{
+   Bool ret = true;
+   Font font;
+   int i, j, h, w, lineSize;
+   HBITMAP bm, oldBM;
+   BITMAPINFO bi;
+   HDC dc;
+   LOGFONT logfont;
+   HFONT hfont, oldFont;
+   Byte * glyph, * empty;
+
+   /* don't to need exact dimension, just to fit the glyph is enough */
+   w = var font. maximalWidth * 2;
+   h = var font. height;
+   lineSize = (( w + 31) / 32) * 4;
+   w = lineSize * 8;
+   if ( !( glyph = malloc( lineSize * ( h + 1 ))))
+      return false;
+   empty = glyph + lineSize * h;
+   memset( empty, 0xff, lineSize);
+
+   if ( !( dc = CreateCompatibleDC( NULL ))) {
+      free( glyph );
+      return false;
+   }
+   if ( !( bm = CreateBitmap( w, h, 1, 1, NULL))) {
+      free( glyph );
+      return false;
+   }
+
+   bi. bmiHeader. biSize         = sizeof( bi. bmiHeader);
+   bi. bmiHeader. biPlanes       = 1;
+   bi. bmiHeader. biBitCount     = 1;
+   bi. bmiHeader. biSizeImage    = lineSize * h;
+   bi. bmiHeader. biWidth        = w;
+   bi. bmiHeader. biHeight       = h;
+   bi. bmiHeader. biCompression  = BI_RGB;
+   bi. bmiHeader. biClrUsed      = 2;
+   bi. bmiHeader. biClrImportant = 2;
+
+   oldBM    = SelectObject( dc, bm );
+
+   font = var font;
+   font. direction = 0;
+   font_font2logfont( &font, &logfont);
+   hfont = CreateFontIndirect( &logfont);
+   oldFont = SelectObject( dc, hfont );
+
+   memset( abc, 0, sizeof(FontABC) * (last - first + 1));
+   for ( i = 0; i <= last - first; i++) {
+      Rectangle( dc, -1, -1, w+2, h+2 );
+      if ( unicode ) {
+         WCHAR ch = first + i;
+         TextOutW( dc, var font. maximalWidth, 0, &ch, 1);
+      } else {
+         CHAR ch = first + i;
+         TextOutA( dc, var font. maximalWidth, 0, &ch, 1);
+      }
+
+      if ( !GetDIBits( dc, bm, 0, h, glyph, &bi, DIB_RGB_COLORS)) {
+         ret = false;
+         break;
+      }
+
+/*
+      for ( j = 0; j < h; j++) {
+         int k, l;
+         for ( k = 0; k < lineSize; k++) {
+            Byte * p = glyph + j * lineSize + k;
+            printf(".");
+            for ( l = 0; l < 8; l++) {
+               int z = (*p) & ( 1 << (7-l) );
+               printf("%s", z ? "*" : " ");
+            }
+         }
+         printf("\n");
+      }
+*/      
+
+      for ( j = 0; j < h; j++) {
+         if ( memcmp( glyph + j * lineSize, empty, lineSize) != 0 ) {
+            abc[i]. a = j;
+            break;
+         }
+      }
+      for ( j = h - 1; j >= 0; j--) {
+         if ( memcmp( glyph + j * lineSize, empty, lineSize) != 0 ) {
+            abc[i]. c = h - j - 1;
+            break;
+         }
+      }
+
+      if ( abc[i]. a != 0 || abc[i].c != 0)
+         abc[i]. b = h - abc[i]. a - abc[i]. c;
+   }
+
+   SelectObject( dc, oldFont);
+   SelectObject( dc, oldBM );
+   DeleteObject( hfont );
+   DeleteObject( bm );
+   DeleteDC( dc );
+   free( glyph );
+   return true;
+}
+
+PFontABC
+apc_gp_get_font_def( Handle self, int first, int last, Bool unicode)
+{objCheck nil;{
+   int i;
+   DWORD ret;
+   PFontABC f1;
+   MAT2 gmat = { {0, 1}, {0, 0}, {0, 0}, {0, 1} };
+   GLYPHMETRICS g;
+   char buf[sizeof(GLYPHMETRICS)];
+
+   f1 = ( PFontABC) malloc(( last - first + 1) * sizeof( FontABC));
+   if ( !f1) return nil;
+
+   for ( i = 0; i <= last - first; i++) {
+      memset(&g, 0, sizeof(g));
+      if ( unicode ) {
+         ret = GetGlyphOutlineW(sys ps, i + first, GGO_METRICS, &g, sizeof(g), NULL, &gmat);
+      } else {
+         ret = GetGlyphOutlineA(sys ps, i + first, GGO_METRICS, &g, sizeof(g), NULL, &gmat);
+      }
+      if ( ret == GDI_ERROR ) {
+         if ( !gp_get_font_def_bitmap( self, first, last, unicode, f1 )) {
+	    free( f1 );
+	    return nil;
+	 }
+	 return f1;
+      }
+      f1[i]. a = var font. descent + g.gmptGlyphOrigin. y - g.gmBlackBoxY; /* XXX g.gmCellIncY ? */
+      f1[i]. b = g.gmBlackBoxY;
+      f1[i]. c = var font.ascent - g.gmptGlyphOrigin. y;
+   }
+
+   return f1;
+}}
+
 static Handle ipa_ranges[] = {
   0x0250 , 0x02AF,	// 4  IPA Extensions
   0x1D00 , 0x1D7F,  //    Phonetic Extensions
@@ -1431,7 +1368,7 @@ apc_gp_get_region( Handle self, Handle mask)
    PaintRgn( dc, rgn);
    DeleteObject( SelectObject( dc, brSave));
 
-   bi = image_get_binfo( mask, &xbi);
+   bi = image_fill_bitmap_info( mask, &xbi, BM_BITMAP);
    if ( !GetDIBits( dc, bm, 0, PImage( mask)-> h, PImage( mask)-> data, bi, DIB_RGB_COLORS)) apiErr;
    SelectObject( dc, bmSave);
    DeleteObject( bm);
@@ -1525,9 +1462,11 @@ gp_get_text_width( Handle self, const char* text, int len, Bool addOverhang, Boo
       if ( sys tmPitchAndFamily & TMPF_TRUETYPE) {
          ABC abc[2];
          if ( wide) {
-	    /* the 1st character is always 0x202D, skip it */ 
-	    text++; 
-	    len--;
+	    if ( guts. utf8_prepend_0x202D ) {
+	       /* the 1st character is 0x202D, skip it */
+	       text++;
+	       len--;
+	    }
 
             GetCharABCWidthsW( sys ps, *((WCHAR*)text), *((WCHAR*)text), &abc[0]);
             GetCharABCWidthsW( sys ps, *((WCHAR*)text + len - 1), *((WCHAR*)text + len - 1), &abc[1]);
@@ -1588,6 +1527,11 @@ apc_gp_get_text_box( Handle self, const char* text, int len, Bool utf8)
    if ( sys tmPitchAndFamily & TMPF_TRUETYPE) {
       ABC abc[2];
       if ( utf8) {
+	 if ( guts. utf8_prepend_0x202D ) {
+	    /* the 1st character is 0x202D, skip it */
+	    text++;
+	    len--;
+	 }
          GetCharABCWidthsW( sys ps, *((WCHAR*)text), *((WCHAR*)text), &abc[0]);
          GetCharABCWidthsW( sys ps, *((WCHAR*)text + len - 1), *((WCHAR*)text + len - 1), &abc[1]);
       } else {
