@@ -701,9 +701,11 @@ apc_font_default( PFont font)
 }
 
 int
-apc_font_load( const char* filename)
+apc_font_load( Handle self, char* filename)
 {
-   return 0;
+   int ret = AddFontResource(( LPCTSTR) filename );
+   if (ret) hash_store( myfontMan, filename, strlen(filename), (void*)NULL);
+   return ret;
 }
 
 #define fgBitmap 0
@@ -1532,7 +1534,7 @@ hwnd_leave_paint( Handle self)
    stylus_free( sys stylusResource, false);
    if ( sys opaquePen ) {
    	DeleteObject( sys opaquePen );
-	sys opaquePen = nil;
+      sys opaquePen = nil;
    }
    if ( sys psd != nil) {
       var font           = sys psd-> font;
@@ -1553,6 +1555,25 @@ hwnd_leave_paint( Handle self)
    sys bpp = 0;
 }
 
+Bool
+hwnd_repaint_layered( Handle self, Bool now )
+{
+   Event ev;
+   if ( !is_apt( aptLayered)) return;
+
+   if ( !now && !is_apt( aptSyncPaint) ) {
+      if ( !is_apt( aptRepaintPending )) {
+         apt_set( aptRepaintPending );
+         PostMessage(( HWND) var handle, WM_REPAINT_LAYERED, 0, 0);
+      }
+      return;
+   }
+ 
+   apt_clear( aptRepaintPending );
+   ev. cmd = cmPaint;
+   CWidget(self)-> message( self, &ev); 
+}
+
 Handle
 hwnd_top_level( Handle self)
 {
@@ -1574,7 +1595,6 @@ hwnd_frame_top_level( Handle self)
    return nilHandle;
 }
 
-
 typedef struct _ModData
 {
    BYTE  ks  [ 256];
@@ -1841,9 +1861,9 @@ region_create( Handle mask)
 
    w = PImage( mask)-> w;
    h = PImage( mask)-> h;
-   if ( dsys( mask) s. imgCachedRegion) {
+   if ( dsys( mask) s. image. imgCachedRegion) {
       rgn = CreateRectRgn(0,0,0,0);
-      CombineRgn( rgn, dsys( mask) s. imgCachedRegion, nil, RGN_COPY);
+      CombineRgn( rgn, dsys( mask) s. image. imgCachedRegion, nil, RGN_COPY);
       return rgn;
    }
 
@@ -1889,6 +1909,7 @@ region_create( Handle mask)
       idata -= PImage( mask)-> lineSize;
    }
 
+
    if ( set) {
       rdata-> rdh. dwSize          = sizeof( RGNDATAHEADER);
       rdata-> rdh. iType           = RDH_RECTANGLES;
@@ -1903,8 +1924,10 @@ region_create( Handle mask)
          apcErr( 900);
       }
 
-      dsys( mask) s. imgCachedRegion = CreateRectRgn(0,0,0,0);
-      CombineRgn( dsys( mask) s. imgCachedRegion, rgn, nil, RGN_COPY);
+      dsys( mask) s. image. imgCachedRegion = CreateRectRgn(0,0,0,0);
+      CombineRgn( dsys( mask) s. image. imgCachedRegion, rgn, nil, RGN_COPY);
+   } else {
+      dsys( mask) s. image. imgCachedRegion = rgn = CreateRectRgn(0,0,0,0);
    }
    free( rdata);
 
